/*********************************************************************
 *
 * Authors: Vincenzo Ciaschini - Vincenzo.Ciaschini@cnaf.infn.it 
 *
 * Copyright (c) 2002, 2003 INFN-CNAF on behalf of the EU DataGrid.
 * For license conditions see LICENSE file or
 * http://www.edg.org/license.html
 *
 * Parts of this code may be based upon or even include verbatim pieces,
 * originally written by other people, in which case the original header
 * follows.
 *
 *********************************************************************/

/* $Revision: 1.14 $ */
/* $Date: 2003/10/27 09:27:58 $ */

#ifndef _VOMS_API_H
#define _VOMS_API_H

#include <string>
#include <vector>
#include <openssl/x509.h>

#include <sys/types.h>

/*! \brief User's characteristics: can be repeated. 
*/
struct data {
  std::string group; /*!< user's group */
  std::string role;  /*!< user's role */
  std::string cap;   /*!< user's capability */
};

/*! \brief The type of data returned.
 */
enum data_type { 
  TYPE_NODATA,  /*!< no data */
  TYPE_STD,     /*!< group, role, capability triplet */
  TYPE_CUSTOM   /*!< result of an S command */
};

struct contactdata {   /*!< You must never allocate directly this structure. 
			    Its sizeof() is subject to change without notice. 
			    The only supported way to obtain it is via the 
			    FindBy* functions. */
  std::string nick;    /*!< The alias of the server */
  std::string host;    /*!< The hostname of the server */
  std::string contact; /*!< The subject of the server's certificate */
  std::string vo;      /*!< The VO served by this server */
  int    port;	       /*!< The port on which the server is listening */
};			
			
struct voms {
  int siglen;         /*!< The length of the VOMS server signature */
  std::string signature;   /*!< The VOMS server signature */
  std::string user;        /*!< The user's DN, as from his certificate */
  std::string userca;      /*!< The CA which signed the user's certificate */
  std::string server;      /*!< The VOMS server DN, as from its certificate */
  std::string serverca;    /*!< The CA which signed the VOMS certificate */
  std::string voname;      /*!< The name of the VO to which the VOMS belongs */
  std::string uri;         /*!< The URI of the VOMS server */
  std::string date1;       /*!< Beginning of validity of the user info */
  std::string date2;       /*!< End of validity of the user info */
  data_type type;     /*!< The type of data returned */
  std::vector<data> std;   /*!< User's characteristics */
  std::string custom;      /*!< The data returned by an S command */
};

enum recurse_type { 
  RECURSE_CHAIN, 
  RECURSE_NONE 
};



/*! \brief Error codes.
*/
enum verror_type { 
  VERR_NONE,
  VERR_NOSOCKET,  /*!< Socket problem*/
  VERR_NOIDENT,   /*!< Cannot identify itself (certificate problem) */
  VERR_COMM,      /*!< Server problem */
  VERR_PARAM,     /*!< Wrong parameters*/
  VERR_NOEXT,     /*!< VOMS extension missing */
  VERR_NOINIT,    /*!< Initialization error */
  VERR_TIME,      /*!< Error in time checking */
  VERR_IDCHECK,   /*!< User data in extension different from the real ones */
  VERR_EXTRAINFO, /*!< VO name and URI missing */
  VERR_FORMAT,    /*!< Wrong data format */
  VERR_NODATA,    /*!< Empty extension */
  VERR_PARSE,     /*!< Parse error */
  VERR_DIR,       /*!< Directory error */
  VERR_SIGN,      /*!< Signature error */
  VERR_SERVER,    /*!< Unidentifiable VOMS server */
  VERR_MEM,       /*!< Memory problems */
  VERR_VERIFY,    /*!< Generic verification error*/
  //  VERR_IDENT, 
  VERR_TYPE,      /*!< Returned data of unknown type */
  VERR_ORDER      /*!< Ordering different than required */
};


struct vomsdata {
  private:
  std::string ca_cert_dir;
  std::string voms_cert_dir;
  int duration;
  std::string ordering;
  std::vector<contactdata> servers;

  public:
  verror_type error; /*!< Error code */

  vomsdata(std::string voms_dir = "", 
	   std::string cert_dir = ""); /*!< \param voms_dir The directory which contains the certificate 
                                              of the VOMS server
                                       \param cert_dir The directory which contains the certificate of the CA

                                       If voms_dir is empty, the value of the environment variable
                                       X509_VOMS_DIR is taken.

                                       If cert_dir is empty, the value of the environment variable
                                       X509_CERT_DIR is taken.
                                   */

  bool LoadSystemContacts(std::string dir = ""); /*!< Loads the system wide configuration files.
						   \param dir The directory in which the files are stored.

						   If dir is empty, defaults to /etc/voms/vomses.

						   \return True if all went OK, false otherwise.
						 */
  bool LoadUserContacts(std::string dir = ""); /*!< Loads the user-specific configuration files.
						 \param dir The directory in which the files are stored.

						 If dir is empty, defaults to $VOMS_USERCONF. If this is 
						 empty too, defaults to $HOME/.edg/vomses, or to
						 ~/.edg/vomses as a last resort.

						 \return True if all went OK, false otherwise.
						 */

  std::vector<contactdata> FindByAlias(std::string alias); /*!< Finds servers which share a common alias.
							     \param alias The alias to look for.

							     \return The servers found. The order in which
							             they are returned is unspecified.
							   */


  std::vector<contactdata> FindByVO(std::string vo); /*!< Finds servers which serve a common VO
						       \param vo The VO name to look for.

						       \return The servers found. The order in which
						               they are returned is unspecified.
						     */


  void Order(std::string att); /*!< Sets up the ordering of the results.

			    Defines the ordering of the data returned by Contact(). Results are
			    ordered in the same order as the calls to this function.
			    \param att The attribute to be ordered.
			  */

  void ResetOrder(void); /*!< Resets the ordering. */

  bool Retrieve(X509 *cert, STACK_OF(X509) *chain, 
		recurse_type how = RECURSE_CHAIN); /*!< Extracts the VOMS extension from an X.509 certificate. 
                                                     The function doesn't check the validity of the certificates, 
                                                     but it does check the content of the user data.
                                                     \param cert The certificate with the VOMS extensions
                                                     \param chain The chain of the validation certificates 
                                                           (only the intermediate ones)
                                                     \param how Recursion type
                                                     \return failure (F) or success (T)
                                                   */  
  bool Contact(std::string hostname, 
               int port, 
               std::string servsubject, 
               std::string command); /*!< Contacts a VOMS server to get a certificate

                                    It is the equivalent of the voms_proxy_init command, but 
                                    without the --include functionality.
                                    \param hostname FQDN of the VOMS server
                                    \param port the port on which the VOMS server is listening
                                    \param servsubject the subject of the server's certificate
                                    \param command Command
                                    \return failure (F) or success (T)
				*/
  void SetLifetime(int lifetime); /*!< Set requested lifetime for the Contact() call.
				    \param lifetime Requested lifetime, in seconds
				  */

  bool Import(std::string buffer);/*!< Converts data from the format used for inclusion 
                                  into a certificate to the internal format

                                     The function does verify the data.
                                     \param buffer contains the data to be converted 
                                     \return Failure (F) or Success (T)
                             */
  bool Export(std::string &data); /*!< Exports data from vomsdata::data to the format 
                                  used for inclusion into a certificate. 

                                  The function doesn't verify the data
                                  \param data The certificate extension
                                  \return Failure (F) or Success (T)
                             */
  bool DefaultData(voms &); /*!< Get the default data extension from those present in
			      the pseudo certificate */

  std::vector<voms> data; /*!< User's info, as in the certificate extension.
                          It may contain data gathered from more than one VOMS server, 
                     */
  std::string workvo;     /*!< The value of the -vo option of the voms-proxy-init command */
  std::string extra_data; /*!< The data specified by the user with the --include switch.
 
                          Note that this field doesn't contain the result of a request
                          to the VOMS server, but instead data specified by the user.

                          The reason for the introduction of this extension is to let 
                          a user include important data into his proxy certificate, 
                          like, for example, a kerberos ticket 
                      */
private:
  bool loadfile(std::string, uid_t uid, gid_t gid);
  bool loadfile0(std::string, uid_t uid, gid_t gid);
};


int getMajorVersionNumber(void);
int getMinorVersionNumber(void);
int getPatchVersionNumber(void);

#endif
